package com.ozs.plan.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.core.domain.entity.SysDictData;
import com.ozs.common.core.domain.entity.SysRole;
import com.ozs.common.core.domain.entity.SysUser;
import com.ozs.common.core.domain.model.LoginUser;
import com.ozs.common.enums.DataIsDelete;
import com.ozs.common.enums.IsExcess;
import com.ozs.common.enums.PmProjectStatus;
import com.ozs.common.enums.ProjectStatus;
import com.ozs.common.enums.ProjectTypes;
import com.ozs.common.enums.SysFileRefEnum;
import com.ozs.common.exception.ServiceException;
import com.ozs.common.utils.PageUtils;
import com.ozs.common.utils.SecurityUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.bean.BeanUtils;
import com.ozs.home.domain.vo.HomeToDoQueryResVo;
import com.ozs.plan.doman.PlanQuarter;
import com.ozs.plan.doman.PlanYears;
import com.ozs.plan.doman.vo.requestVo.PlanYearsStandardVo;
import com.ozs.plan.doman.vo.responseVo.PlanQuarterResponseVo;
import com.ozs.plan.doman.vo.responseVo.PlanYearsResponseVo;
import com.ozs.plan.mapper.PlanQuarterMapper;
import com.ozs.plan.mapper.PlanYearsMapper;
import com.ozs.plan.service.PlanYearsService;
import com.ozs.pm.doman.PmDemand;
import com.ozs.pm.doman.vo.responseVo.PmDemandResVo;
import com.ozs.pm.mapper.PmDemandMapper;
import com.ozs.pm.service.IPmDemandService;
import com.ozs.system.domain.SysFileInfo;
import com.ozs.system.domain.SysFileRef;
import com.ozs.system.domain.vo.responseVo.SysDeptResponseVo;
import com.ozs.system.mapper.SysDeptMapper;
import com.ozs.system.mapper.SysFileRefMapper;
import com.ozs.system.mapper.SysRoleMapper;
import com.ozs.system.service.ISysDeptService;
import com.ozs.system.service.ISysDictTypeService;
import com.ozs.system.service.ISysUserService;
import com.ozs.system.service.SysFileService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import javax.validation.Validator;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author buzhanyi
 */
@Service
@Slf4j
public class PlanYearsServiceImpl extends ServiceImpl<PlanYearsMapper, PlanYears> implements PlanYearsService {

    @Autowired
    PlanYearsMapper planYearsMapper;
    @Autowired
    private PlanQuarterMapper planQuarterMapper;
    @Autowired
    protected Validator validator;
    @Autowired
    private ISysDictTypeService dictTypeService;
    @Autowired
    private ISysDeptService deptService;
    @Autowired
    private SysFileRefMapper sysFileRefMapper;
    @Autowired
    private SysDeptMapper deptMapper;
    @Autowired
    private SysFileService fileService;
    @Autowired
    private SysRoleMapper sysRoleMapper;
    @Autowired
    private ISysUserService userService;
    @Autowired
    private PmDemandMapper pmDemandMapper;
    @Autowired
    private PlanQuarterServiceImpl quartzServiceImpl;
    @Autowired
    private IPmDemandService demandService;

    @Override
    public Page selectPlanYearsList(PlanYearsStandardVo vo) {
        PlanYears ofYears = new PlanYears();
        List<PlanYears> planYears;
        List<PlanYearsResponseVo> planYearsList = new ArrayList<>();
        try {
            BeanUtils.copyProperties(vo, ofYears);
            planYears = planYearsMapper.selectPlanYearsList(ofYears);
            planYearsList = changeTo(planYears);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Page pages = PageUtils.getPages(vo.getPageNum().intValue(), vo.getPageSize().intValue(), planYearsList);
        return pages;
    }

    @Override
    public List<PlanYearsResponseVo> selectPlanYearsListEXP(PlanYearsStandardVo vo) {
        List<PlanYears> list = new ArrayList<>();
        List<PlanYearsResponseVo> planYearsList = new ArrayList<>();
        try {
            int num = 1;
            int size = 200;
            while (size == 200) {
                LambdaQueryWrapper<PlanYears> lw = new LambdaQueryWrapper<PlanYears>();
                if (!ObjectUtils.isEmpty(vo.getProjectName())) {
                    lw.like(PlanYears::getProjectName, vo.getProjectName());
                }
                if (!ObjectUtils.isEmpty(vo.getPurchaseServices())) {
                    lw.eq(PlanYears::getPurchaseServices, vo.getPurchaseServices());
                }
                if (!ObjectUtils.isEmpty(vo.getIsExcess())) {
                    lw.eq(PlanYears::getIsExcess, vo.getIsExcess());
                }
                if (!ObjectUtils.isEmpty(vo.getProjectStatus())) {
                    lw.eq(PlanYears::getProjectStatus, vo.getProjectStatus());
                }
                if (!ObjectUtils.isEmpty(vo.getBeginTime())) {
                    lw.ge(PlanYears::getPlanDemandSubTime, vo.getBeginTime());
                }
                if (!ObjectUtils.isEmpty(vo.getEndTime())) {
                    lw.le(PlanYears::getPlanDemandSubTime, vo.getEndTime());
                }
                lw.eq(PlanYears::getDelFlay, DataIsDelete.DataNOTDelete);
                Page<PlanYears> page = planYearsMapper.selectPage(new Page<PlanYears>(num, size, false), lw);
                list.addAll(page.getRecords());
                size = page.getRecords().size();
                num++;
            }
            planYearsList = changeTo(list);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return planYearsList;
    }

    @Override
    public Page selectPlanYearsExamineList(PlanYearsStandardVo vo) {
        PlanYears ofYears = new PlanYears();
        List<PlanYears> planYears;
        List<PlanYearsResponseVo> planYearsList = new ArrayList<>();
        try {
            BeanUtils.copyProperties(vo, ofYears);
            planYears = planYearsMapper.selectPlanYearsExamineList(ofYears);
            planYearsList = changeTo(planYears);
        } catch (Exception e) {
            e.printStackTrace();
        }
        Page pages = PageUtils.getPages(vo.getPageNum().intValue(), vo.getPageSize().intValue(), planYearsList);
        return pages;
    }

    @Override
    public List<PlanYearsResponseVo> selectPlanYearsExamineListEXP(PlanYearsStandardVo vo) {
        List<PlanYears> list = new ArrayList<>();
        List<PlanYearsResponseVo> planYearsList = new ArrayList<>();
        try {
            int num = 1;
            int size = 200;
            while (size == 200) {
                LambdaQueryWrapper<PlanYears> lw = new LambdaQueryWrapper<PlanYears>();
                if (!ObjectUtils.isEmpty(vo.getProjectName())) {
                    lw.like(PlanYears::getProjectName, vo.getProjectName());
                }
                if (!ObjectUtils.isEmpty(vo.getPurchaseServices())) {
                    lw.eq(PlanYears::getPurchaseServices, vo.getPurchaseServices());
                }
                if (!ObjectUtils.isEmpty(vo.getIsExcess())) {
                    lw.eq(PlanYears::getIsExcess, vo.getIsExcess());
                }
                if (!ObjectUtils.isEmpty(vo.getProjectStatus())) {
                    lw.eq(PlanYears::getProjectStatus, vo.getProjectStatus());
                } else {
                    lw.in(PlanYears::getProjectStatus, "1,3");
                }
                if (!ObjectUtils.isEmpty(vo.getBeginTime())) {
                    lw.ge(PlanYears::getPlanDemandSubTime, vo.getBeginTime());
                }
                if (!ObjectUtils.isEmpty(vo.getEndTime())) {
                    lw.le(PlanYears::getPlanDemandSubTime, vo.getEndTime());
                }
                lw.eq(PlanYears::getDelFlay, DataIsDelete.DataNOTDelete);
                Page<PlanYears> page = planYearsMapper.selectPage(new Page<PlanYears>(num, size, false), lw);
                list.addAll(page.getRecords());
                size = page.getRecords().size();
                num++;
            }
            planYearsList = changeTo(list);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return planYearsList;
    }

    @Transactional
    @Override
    public AjaxResult insertPlanYears(PlanYearsStandardVo yearsStandardVo) {
        if (planYearsMapper.countProjectName(yearsStandardVo.getProjectName()) > 0) {
            return AjaxResult.error("该项目名称已经存在");
        }
        PlanYears ofYears = new PlanYears();
        try {
            BeanUtils.copyProperties(yearsStandardVo, ofYears);
        } catch (Exception e) {
            e.printStackTrace();
        }
        //判断是否为超额计划
        ofYears.setIsExcess(isExcessOrNo(ofYears.getProjectType(), ofYears.getEvaluation()));
        ofYears.setProjectStatus(ProjectStatus.PLANWAITCOMMIT.getCode());
        ofYears.setCreateTime(new Date());
        planYearsMapper.insertPlanYears(ofYears);
        log.info("id:{}", ofYears.getPlanYearId());
        List<SysFileRef> sysFileRefs = yearsStandardVo.getSysFileRefs();
        if (!ObjectUtils.isEmpty(sysFileRefs)) {
            for (SysFileRef ref : sysFileRefs) {
                ref.setRedId(ofYears.getPlanYearId());
                ref.setType(SysFileRefEnum.PLAN_YEAR.getType());
                ref.setCreated(ofYears.getCreated());
                ref.setCreateTime(new Date());
                ref.setUpdated(ofYears.getCreated());
                ref.setUpdateTime(new Date());
                sysFileRefMapper.insert(ref);
            }
        }
        return AjaxResult.success();
    }

    @Override
    public AjaxResult deletePlanYearsByIds(Long[] planIds) {
        planYearsMapper.deletePlanYearsByIds(planIds);
        return AjaxResult.success();
    }

    @Override
    public AjaxResult deletePlanYearsById(Long planId) {
        planYearsMapper.deletePlanYearsById(planId);
        return AjaxResult.success();
    }

    @Override
    public AjaxResult view(PlanYearsStandardVo yearsStandardVo) {

        PlanYearsResponseVo responseVo = new PlanYearsResponseVo();
        PlanYears byId = planYearsMapper.getById(yearsStandardVo.getPlanYearId());
        byId.getPlanYearId();
        if (byId == null) {
            return AjaxResult.error("数据查询失败");
        }
        SysDeptResponseVo sysDeptResponseVo = (SysDeptResponseVo) deptService.selectDeptById(Long.valueOf(byId.getPurchaseDeptId())).get("sysDept");
        byId.setPurchaseDeptName(sysDeptResponseVo.getDeptName());
        HashMap<String, Object> map = new HashMap<>();
        map.put("red_id", byId.getPlanYearId());
        map.put("type", SysFileRefEnum.PLAN_YEAR.getType());
        List<SysFileRef> fileRefs = sysFileRefMapper.selectByMap(map);
        List<SysFileInfo> fileInfos = new ArrayList<>();
        BeanUtils.copyProperties(byId, responseVo);
        if (!ObjectUtils.isEmpty(fileRefs)) {
            for (SysFileRef ref : fileRefs) {
                SysFileInfo fileInfo = fileService.getById(ref.getFileId());
                fileInfos.add(fileInfo);
            }
            responseVo.setFileInfos(fileInfos);
        }
        return AjaxResult.success(responseVo);
    }

    @Transactional
    @Override
    public AjaxResult update(PlanYearsStandardVo yearsStandardVo) {
        if (planYearsMapper.countProjectNameOther(yearsStandardVo.getProjectName(), String.valueOf(yearsStandardVo.getPlanYearId())) > 0) {
            return AjaxResult.error("该项目名称已经存在");
        }
        PlanYears ofYears = new PlanYears();
        try {
            BeanUtils.copyProperties(yearsStandardVo, ofYears);
        } catch (Exception e) {
            e.printStackTrace();
        }
        ofYears.setIsExcess(isExcessOrNo(ofYears.getProjectType(), ofYears.getEvaluation()));
        ofYears.setUpdateTime(new Date());
        planYearsMapper.updateInfoById(ofYears);
        List<SysFileRef> sysFileRefs = yearsStandardVo.getSysFileRefs();
        HashMap<String, Object> map = new HashMap<>();
        map.put("red_id", ofYears.getPlanYearId());
        map.put("type", "1");
        sysFileRefMapper.deleteByMap(map);
        if (!ObjectUtils.isEmpty(sysFileRefs)) {
            for (SysFileRef ref : sysFileRefs) {
                ref.setRedId(ofYears.getPlanYearId());
                ref.setType(SysFileRefEnum.PLAN_YEAR.getType());
                ref.setCreated(yearsStandardVo.getUpdated());
                ref.setCreateTime(new Date());
                ref.setUpdated(yearsStandardVo.getUpdated());
                ref.setUpdateTime(new Date());
                sysFileRefMapper.insert(ref);
            }
        }
        return AjaxResult.success();
    }

    @Transactional
    @Override
    public String importPlanYears(List<PlanYearsStandardVo> planYears, boolean isUpdateSupport, LoginUser loginUser) {
        if (StringUtils.isNull(planYears) || planYears.size() == 0) {
            throw new ServiceException("导入年度计划数据不能为空！");
        }
        List<PlanYears> plans = new ArrayList<>();
        for (PlanYearsStandardVo yearsStandardVo : planYears) {
            PlanYears ofYears = new PlanYears();
            BeanUtils.copyProperties(yearsStandardVo, ofYears);
            ofYears.setPurchaseDeptName(yearsStandardVo.getPurchaseDeptId());
            plans.add(ofYears);
        }
        HashMap<String, HashMap<String, String>> planEnums = dictTypeService.getAboutEnums();
        HashMap<String, String> projectTypesMap = planEnums.get("projectTypes");
        HashMap<String, String> planPurchaseModesMap = planEnums.get("planPurchaseModes");
        HashMap<String, String> projectAttributes = planEnums.get("projectAttributes");
        HashMap<String, String> purchaseServices = planEnums.get("purchaseServices");

        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        //deptService
        for (PlanYears ofYear : plans) {
            try {
                //验证项目名称是否重复导入
                //将录入信息中的值更改为要保存的数据
                if (planYearsMapper.countProjectName(ofYear.getProjectName()) == 0) {
                    //采购单位
                    SysDept info = deptMapper.checkDeptNameOnlyOne(ofYear.getPurchaseDeptName());
                    if (!ObjectUtils.isEmpty(info)) {
                        ofYear.setPurchaseDeptId(String.valueOf(info.getDeptId()));
                    } else {
                        ++failureNum;
                        failureMsg.append("*" + failureNum + "*采购单位“" + ofYear.getPurchaseDeptName() + "”不存在");
                    }
                    ofYear.setProjectType(projectTypesMap.get(ofYear.getProjectType()));
                    ofYear.setProjectStatus(ProjectStatus.PLANWAITCOMMIT.getCode());
                    ofYear.setPurchaseMode(planPurchaseModesMap.get(ofYear.getPurchaseMode()));
                    //项目属性是多选字段
                    StringBuilder builder = new StringBuilder();
                    if (ofYear.getProjectAttr().contains(",")) {
                        String[] split = ofYear.getProjectAttr().split(",");
                        for (String s : split) {
                            if (!ofYear.getProjectAttr().endsWith(s)) {
                                builder.append(projectAttributes.get(s) + ",");
                            } else {
                                builder.append(projectAttributes.get(s));
                            }
                        }
                    } else {
                        builder.append(projectAttributes.get(ofYear.getProjectAttr()));
                    }
                    ofYear.setProjectAttr(builder.toString());
                    ofYear.setPurchaseServices(purchaseServices.get(ofYear.getPurchaseServices()));
                    ofYear.setIsExcess(isExcessOrNo(ofYear.getProjectType(), ofYear.getEvaluation()));
                    ofYear.setCreated(String.valueOf(loginUser.getUserId()));
                    ofYear.setCreateTime(new Date());
                    planYearsMapper.insertPlanYears(ofYear);
                    successNum++;
                    successMsg.append("*" + successNum + "、项目 " + ofYear.getProjectName() + " 导入成功!");
                    //} else if (isUpdateSupport) {
                    //    PlanYears years = plan.get(0);
                    //    BeanValidators.validateWithException(validator, ofYear);
                    //    years.setCreateTime(new Date());
                    //    years.setProjectStatus(ProjectStatus.PLANWAITCOMMIT.getCode());
                    //    planYearsMapper.updateById(years);
                    //    successNum++;
                    //    successMsg.append("<br/>" + successNum + "、项目 " + ofYear.getProjectName() + " 更新成功");
                } else {
                    failureNum++;
                    failureMsg.append(failureNum + "*项目 " + ofYear.getProjectName() + " 已存在！/n");
                }
            } catch (Exception exc) {
                failureNum++;
                String msg = "*" + successNum + "*项目 " + ofYear.getProjectName() + " 导入失败";
                failureMsg.append(msg + exc.getMessage());
                log.error(msg, exc);
            }
        }
        if (failureNum > 0) {
            failureMsg.insert(0, "导入失败！共 " + failureNum + " 条数据格式不正确：");
            throw new ServiceException(failureMsg.toString());
        } else {
            successMsg.insert(0, "导入成功！共 " + successNum + " 条。");
        }
        return successMsg.toString();
    }

    @Override
    public AjaxResult commit(PlanYearsStandardVo yearsStandardVo) {
        //PlanYears byId = PlanYearsMapper.getById(yearsStandardVo.getPlanYearId());
        //BigDecimal evaluation = byId.getEvaluation();
        //BigDecimal threshold = new BigDecimal(0);
        ////获取各个项目类型设定的概算金额阈值
        //List<SysDictData> data = dictTypeService.selectDictDataByType("sys_over_limit_threshold");
        //HashMap<String, String> thresholdMap = new LinkedHashMap<>();
        ////各个类型的概算金额阈值
        //for (SysDictData dictData : data) {
        //    //    类型----阈值
        //    thresholdMap.put(dictData.getDictLabel(), dictData.getDictValue());
        //}
        ////项目类型
        //for (ProjectTypes value : ProjectTypes.values()) {
        //    if (byId.getProjectType().equals(value.getCode())) {
        //        threshold = BigDecimal.valueOf(Long.parseLong(thresholdMap.get(value.getInfo())));
        //    }
        //    break;
        //}
        //
        //if (evaluation.compareTo(threshold) == 1) {
        //    //是超额项目
        //}

        int commit = planYearsMapper.commit(yearsStandardVo.getPlanYearId());
        if (commit != 1) {
            return AjaxResult.error("项目状态数据异常");
        }
        return AjaxResult.success();
    }

    @Override
    @Transactional
    public AjaxResult reviewTo(PlanYearsStandardVo vo) {
        //年度计划审核通过后同步到季度计划和项目管理
        PlanYears ofYears = new PlanYears();
        PlanQuarter quarters = new PlanQuarter();
        PmDemand demand = new PmDemand();
        PlanYears byId = planYearsMapper.getById(vo.getPlanYearId());
        BeanUtils.copyProperties(byId, quarters);
        BeanUtils.copyProperties(byId, demand);

        //同步到季度计划状态为已审核---(已审核过的不再同步)
        PlanQuarter quarter = planQuarterMapper.getByPlanYearId(vo.getPlanYearId());
        if (ObjectUtils.isEmpty(quarter)) {
            quarters.setPlanType(0);
            quarters.setPlanYearId(byId.getPlanYearId());
            quarters.setProjectStatus(ProjectStatus.PLANTOEXAMINE.getCode());
            quarters.setCreateTime(new Date());
            planQuarterMapper.insertPlanQuarter(quarters);
            //同步到项目管理状态为需求待填制
            demand.setPlanId(quarters.getPlanPracticalId());
            demand.setEvaluation(byId.getEvaluation().doubleValue());
            demand.setProjectStatus(PmProjectStatus.DEMAND_WAIT_FILL.getCode());
            demand.setCreateTime(new Date());
            pmDemandMapper.insert(demand);
            //计划附件也进行同步---查询年度计划附件后复制一份改为项目需求附件
            LambdaQueryWrapper<SysFileRef> sysFileRefLp = new LambdaQueryWrapper<>();
            sysFileRefLp.eq(SysFileRef::getType, SysFileRefEnum.PLAN_YEAR.getType());
            sysFileRefLp.eq(SysFileRef::getRedId, vo.getPlanYearId());
            List<SysFileRef> sysFileRefs = sysFileRefMapper.selectList(sysFileRefLp);
            if (!ObjectUtils.isEmpty(sysFileRefs)) {
                for (SysFileRef ref : sysFileRefs) {
                    ref.setRedId(demand.getDemandId());
                    ref.setType(SysFileRefEnum.PM_DEMAND.getType());
                    ref.setId(null);
                    sysFileRefMapper.insert(ref);
                }
            }
        }

        vo.setProjectStatus(ProjectStatus.PLANTOEXAMINE.getCode());
        return review(vo);
    }

    @Override
    @Transactional
    public AjaxResult reviewReturn(PlanYearsStandardVo vo) {
        vo.setProjectStatus(ProjectStatus.PLANTOBACK.getCode());
        return review(vo);
    }

    private AjaxResult review(PlanYearsStandardVo vo) {
        List<SysFileRef> sysFileRefs = vo.getSysFileRefs();
        if (!ObjectUtils.isEmpty(sysFileRefs)) {
            for (SysFileRef ref : sysFileRefs) {
                ref.setRedId(vo.getPlanYearId());
                ref.setType(SysFileRefEnum.PLAN_YEAR_EXAMINE.getType());
                ref.setCreated(vo.getUpdated());
                ref.setCreateTime(new Date());
                ref.setUpdated(vo.getUpdated());
                ref.setUpdateTime(new Date());
                sysFileRefMapper.insert(ref);
            }
        }
        int review = planYearsMapper.review(vo);
        if (review != 1) {
            return AjaxResult.error("项目状态数据异常");
        }
        return AjaxResult.success();
    }

    @Override
    public AjaxResult appUpdate(PlanYearsStandardVo vo) {
        //申请后会成为未提交的状态，可以进行修改
        vo.setProjectStatus(ProjectStatus.PLANWAITCOMMIT.getCode());
        int review = planYearsMapper.review(vo);
        if (review != 1) {
            return AjaxResult.error("项目状态数据异常");
        }
        return AjaxResult.success();
    }

    @Override
    public AjaxResult sendLetter(PlanYearsStandardVo yearsStandardVo) {
        int review = planYearsMapper.sendLetter(yearsStandardVo);
        if (review != 1) {
            return AjaxResult.error("项目状态数据异常");
        }
        return AjaxResult.success();
    }

    @Override
    public AjaxResult upLoadPlanFile(PlanYearsStandardVo yearsStandardVo) {
        List<SysFileRef> sysFileRefs = yearsStandardVo.getSysFileRefs();
        if (!ObjectUtils.isEmpty(sysFileRefs)) {
            for (SysFileRef ref : sysFileRefs) {
                ref.setRedId(yearsStandardVo.getPlanYearId());
                ref.setType(SysFileRefEnum.PLAN_YEAR.getType());
                sysFileRefMapper.insert(ref);
            }
        }
        return AjaxResult.success();
    }

    @Override
    public AjaxResult downLoadPlanFile(PlanYearsStandardVo yearsStandardVo) {

        return AjaxResult.success();
    }

    @Override
    public AjaxResult demandUnitPlanManagementTotal() {
        List<PlanYears> planYears = new ArrayList<>();
        try {
            List<SysRole> sysRoles = sysRoleMapper.selectRolePermissionByUserId(SecurityUtils.getUserId());
            List<SysRole> sysRoleList = sysRoles.stream().filter(tdto -> tdto.getRoleKey().equals("demand_unit")).collect(Collectors.toList());
            if (sysRoleList.size() > 0) {
                SysUser sysUser = userService.selectUserById(SecurityUtils.getUserId());
                SysDept sysDept = deptMapper.selectDeptById(sysUser.getDeptId());
                QueryWrapper<PlanYears> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("purchase_dept_id", sysDept.getDeptId());
                planYears = planYearsMapper.selectList(queryWrapper);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(planYears.size());
    }

    @Override
    public AjaxResult purchasingManagementPurchasingManagementTotal() {
        List<PlanYears> planYears = new ArrayList<>();
        try {
            List<SysRole> sysRoles = sysRoleMapper.selectRolePermissionByUserId(SecurityUtils.getUserId());
            List<SysRole> sysRoleList = sysRoles.stream().filter(tdto -> tdto.getRoleKey().equals("demand_unit")).collect(Collectors.toList());
            if (sysRoleList.size() > 0) {
                SysUser sysUser = userService.selectUserById(SecurityUtils.getUserId());
                SysDept sysDept = deptMapper.selectDeptById(sysUser.getDeptId());
                List<SysDept> sysDepts = deptMapper.selectChildrenDeptById(sysDept.getDeptId());
                List<Long> collect = sysDepts.stream().map(SysDept::getDeptId).collect(Collectors.toList());
                if (collect.size() == 0) {
                    collect.add(-1L);
                }
                QueryWrapper<PlanYears> queryWrapper = new QueryWrapper<>();
                queryWrapper
                        .or(wrapper -> wrapper.lambda().eq(PlanYears::getPurchaseDeptId, sysDept.getDeptId())
                                .eq(PlanYears::getIsExcess, IsExcess.NO_EXCESS.getCode()))
                        .or(wrapper -> wrapper.lambda().in(PlanYears::getPurchaseDeptId, collect)
                                .eq(PlanYears::getIsExcess, IsExcess.EXCESS.getCode()))
                        .in("project_status", ProjectStatus.PLANWAIEXAMINE.getCode(), ProjectStatus.PLANTOEXAMINE.getCode());
                planYears = planYearsMapper.selectList(queryWrapper);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(planYears.size());
    }

    @Override
    public AjaxResult demandUnitDemandManagementTotal() {
        List<PmDemand> pmDemands = new ArrayList<>();
        try {
            List<SysRole> sysRoles = sysRoleMapper.selectRolePermissionByUserId(SecurityUtils.getUserId());
            List<SysRole> sysRoleList = sysRoles.stream().filter(tdto -> tdto.getRoleKey().equals("purchasing_management")).collect(Collectors.toList());
            if (sysRoleList.size() > 0) {
                SysUser sysUser = userService.selectUserById(SecurityUtils.getUserId());
                SysDept sysDept = deptMapper.selectDeptById(sysUser.getDeptId());
                QueryWrapper<PmDemand> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("purchase_dept_id", sysDept.getDeptId());
                pmDemands = pmDemandMapper.selectList(queryWrapper);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(pmDemands.size());
    }

    @Override
    public AjaxResult purchasingManagementDemandManagementTotal() {
        List<PmDemand> pmDemands = new ArrayList<>();
        try {
            List<SysRole> sysRoles = sysRoleMapper.selectRolePermissionByUserId(SecurityUtils.getUserId());
            List<SysRole> sysRoleList = sysRoles.stream().filter(tdto -> tdto.getRoleKey().equals("demand_unit")).collect(Collectors.toList());
            if (sysRoleList.size() > 0) {
                SysUser sysUser = userService.selectUserById(SecurityUtils.getUserId());
                SysDept sysDept = deptMapper.selectDeptById(sysUser.getDeptId());
                List<SysDept> sysDepts = deptMapper.selectChildrenDeptById(sysDept.getDeptId());
                List<Long> collect = sysDepts.stream().map(SysDept::getDeptId).collect(Collectors.toList());
                if (collect.size() == 0) {
                    collect.add(-1L);
                }
                QueryWrapper<PmDemand> queryWrapper = new QueryWrapper<>();
                queryWrapper
                        .or(wrapper -> wrapper.lambda().eq(PmDemand::getPurchaseDeptId, sysDept.getDeptId())
                                .eq(PmDemand::getIsExcess, IsExcess.NO_EXCESS.getCode()))
                        .or(wrapper -> wrapper.lambda().in(PmDemand::getPurchaseDeptId, collect)
                                .eq(PmDemand::getIsExcess, IsExcess.EXCESS.getCode()))
                        .in("project_status", PmProjectStatus.DEMAND_WAIT_AUDIT.getCode(), PmProjectStatus.DEMAND_AUDIT_RETURN.getCode());
                pmDemands = pmDemandMapper.selectList(queryWrapper);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(pmDemands.size());
    }

    @Override
    public AjaxResult demandUnitContractManagementTotal() {
        List<PmDemand> pmDemands = new ArrayList<>();
        try {
            List<SysRole> sysRoles = sysRoleMapper.selectRolePermissionByUserId(SecurityUtils.getUserId());
            List<SysRole> sysRoleList = sysRoles.stream().filter(tdto -> tdto.getRoleKey().equals("purchasing_management")).collect(Collectors.toList());
            if (sysRoleList.size() > 0) {
                SysUser sysUser = userService.selectUserById(SecurityUtils.getUserId());
                SysDept sysDept = deptMapper.selectDeptById(sysUser.getDeptId());
                QueryWrapper<PmDemand> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("purchase_dept_id", sysDept.getDeptId());
                queryWrapper.in("project_status", PmProjectStatus.CONTRACT_WAIT_FILL.getCode(), PmProjectStatus.UNDER_CONSTRUCTION.getCode());
                pmDemands = pmDemandMapper.selectList(queryWrapper);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(pmDemands.size());
    }

    @Override
    public AjaxResult demandUnitConstructionManagementTotal() {
        List<PmDemand> pmDemands = new ArrayList<>();
        try {
            List<SysRole> sysRoles = sysRoleMapper.selectRolePermissionByUserId(SecurityUtils.getUserId());
            List<SysRole> sysRoleList = sysRoles.stream().filter(tdto -> tdto.getRoleKey().equals("purchasing_management")).collect(Collectors.toList());
            if (sysRoleList.size() > 0) {
                SysUser sysUser = userService.selectUserById(SecurityUtils.getUserId());
                SysDept sysDept = deptMapper.selectDeptById(sysUser.getDeptId());
                QueryWrapper<PmDemand> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("purchase_dept_id", sysDept.getDeptId());
                queryWrapper.in("project_status", PmProjectStatus.UNDER_CONSTRUCTION.getCode(), PmProjectStatus.COMPLETION_CONSTRUCTION.getCode());
                pmDemands = pmDemandMapper.selectList(queryWrapper);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(pmDemands.size());
    }

    @Override
    public AjaxResult purchasingManagementTaskReleaseTotal() {
        List<PmDemand> pmDemands = new ArrayList<>();
        try {
            List<SysRole> sysRoles = sysRoleMapper.selectRolePermissionByUserId(SecurityUtils.getUserId());
            List<SysRole> sysRoleList = sysRoles.stream().filter(tdto -> tdto.getRoleKey().equals("demand_unit")).collect(Collectors.toList());
            if (sysRoleList.size() > 0) {
                SysUser sysUser = userService.selectUserById(SecurityUtils.getUserId());
                SysDept sysDept = deptMapper.selectDeptById(sysUser.getDeptId());
                List<SysDept> sysDepts = deptMapper.selectChildrenDeptById(sysDept.getDeptId());
                List<Long> collect = sysDepts.stream().map(SysDept::getDeptId).collect(Collectors.toList());
                if (collect.size() == 0) {
                    collect.add(-1L);
                }
                QueryWrapper<PmDemand> queryWrapper = new QueryWrapper<>();
                queryWrapper
                        .or(wrapper -> wrapper.lambda().eq(PmDemand::getPurchaseDeptId, sysDept.getDeptId())
                                .eq(PmDemand::getIsExcess, IsExcess.NO_EXCESS.getCode()))
                        .or(wrapper -> wrapper.lambda().in(PmDemand::getPurchaseDeptId, collect)
                                .eq(PmDemand::getIsExcess, IsExcess.EXCESS.getCode()))
                        .in("project_status", PmProjectStatus.TASK_WAIT_RELEASE.getCode(),
                                PmProjectStatus.WAIT_SELECT_AGENT.getCode(),
                                PmProjectStatus.WAIT_UPLOAD_BID_FILE.getCode(),
                                PmProjectStatus.WAIT_ANNOUNCEMENT.getCode(),
                                PmProjectStatus.WAIT_OPEN_BID.getCode(),
                                PmProjectStatus.WAIT_BID_ANNOUNCEMENT.getCode(),
                                PmProjectStatus.BIDDING_PUBLICITY.getCode()
                        );
                pmDemands = pmDemandMapper.selectList(queryWrapper);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(pmDemands.size());
    }

    @Override
    public AjaxResult procurementOfficeProcurementExecutionTotal() {
        List<PmDemand> pmDemands = new ArrayList<>();
        try {
            List<SysRole> sysRoles = sysRoleMapper.selectRolePermissionByUserId(SecurityUtils.getUserId());
            List<SysRole> sysRoleList = sysRoles.stream().filter(tdto -> tdto.getRoleKey().equals("demand_unit")).collect(Collectors.toList());
            if (sysRoleList.size() > 0) {
                SysUser sysUser = userService.selectUserById(SecurityUtils.getUserId());
                SysDept sysDept = deptMapper.selectDeptById(sysUser.getDeptId());
                List<SysDept> sysDepts = deptMapper.selectChildrenDeptById(sysDept.getDeptId());
                List<Long> collect = sysDepts.stream().map(SysDept::getDeptId).collect(Collectors.toList());
                if (collect.size() == 0) {
                    collect.add(-1L);
                }
                QueryWrapper<PmDemand> queryWrapper = new QueryWrapper<>();
                queryWrapper
                        .or(wrapper -> wrapper.lambda().eq(PmDemand::getPurchaseDeptId, sysDept.getDeptId())
                                .eq(PmDemand::getIsExcess, IsExcess.NO_EXCESS.getCode()))
                        .or(wrapper -> wrapper.lambda().in(PmDemand::getPurchaseDeptId, collect)
                                .eq(PmDemand::getIsExcess, IsExcess.EXCESS.getCode()))
                        .in("project_status",
                                PmProjectStatus.WAIT_SELECT_AGENT.getCode(),
                                PmProjectStatus.WAIT_UPLOAD_BID_FILE.getCode(),
                                PmProjectStatus.WAIT_ANNOUNCEMENT.getCode(),
                                PmProjectStatus.WAIT_OPEN_BID.getCode(),
                                PmProjectStatus.WAIT_BID_ANNOUNCEMENT.getCode(),
                                PmProjectStatus.BIDDING_PUBLICITY.getCode(),
                                PmProjectStatus.CONTRACT_WAIT_FILL.getCode());
                pmDemands = pmDemandMapper.selectList(queryWrapper);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(pmDemands.size());
    }

    @Override
    public AjaxResult procurementOfficeWinningTheBidTotal() {
        List<PmDemand> pmDemands = new ArrayList<>();
        try {
            List<SysRole> sysRoles = sysRoleMapper.selectRolePermissionByUserId(SecurityUtils.getUserId());
            List<SysRole> sysRoleList = sysRoles.stream().filter(tdto -> tdto.getRoleKey().equals("demand_unit")).collect(Collectors.toList());
            if (sysRoleList.size() > 0) {
                SysUser sysUser = userService.selectUserById(SecurityUtils.getUserId());
                SysDept sysDept = deptMapper.selectDeptById(sysUser.getDeptId());
                List<SysDept> sysDepts = deptMapper.selectChildrenDeptById(sysDept.getDeptId());
                List<Long> collect = sysDepts.stream().map(SysDept::getDeptId).collect(Collectors.toList());
                if (collect.size() == 0) {
                    collect.add(-1L);
                }
                QueryWrapper<PmDemand> queryWrapper = new QueryWrapper<>();
                queryWrapper
                        .or(wrapper -> wrapper.lambda().eq(PmDemand::getPurchaseDeptId, sysDept.getDeptId())
                                .eq(PmDemand::getIsExcess, IsExcess.NO_EXCESS.getCode()))
                        .or(wrapper -> wrapper.lambda().in(PmDemand::getPurchaseDeptId, collect)
                                .eq(PmDemand::getIsExcess, IsExcess.EXCESS.getCode()))
                        .lt("project_status", PmProjectStatus.CONTRACT_WAIT_FILL.getCode());
                pmDemands = pmDemandMapper.selectList(queryWrapper);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(pmDemands.size());
    }

    @Override
    public AjaxResult procurementOfficeFailureToWinTheBidTotal() {
        List<PmDemand> pmDemands = new ArrayList<>();
        try {
            List<SysRole> sysRoles = sysRoleMapper.selectRolePermissionByUserId(SecurityUtils.getUserId());
            List<SysRole> sysRoleList = sysRoles.stream().filter(tdto -> tdto.getRoleKey().equals("demand_unit")).collect(Collectors.toList());
            if (sysRoleList.size() > 0) {
                SysUser sysUser = userService.selectUserById(SecurityUtils.getUserId());
                SysDept sysDept = deptMapper.selectDeptById(sysUser.getDeptId());
                List<SysDept> sysDepts = deptMapper.selectChildrenDeptById(sysDept.getDeptId());
                List<Long> collect = sysDepts.stream().map(SysDept::getDeptId).collect(Collectors.toList());
                if (collect.size() == 0) {
                    collect.add(-1L);
                }
                QueryWrapper<PmDemand> queryWrapper = new QueryWrapper<>();
                queryWrapper
                        .or(wrapper -> wrapper.lambda().eq(PmDemand::getPurchaseDeptId, sysDept.getDeptId())
                                .eq(PmDemand::getIsExcess, IsExcess.NO_EXCESS.getCode()))
                        .or(wrapper -> wrapper.lambda().in(PmDemand::getPurchaseDeptId, collect)
                                .eq(PmDemand::getIsExcess, IsExcess.EXCESS.getCode()))
                        .gt("project_status", PmProjectStatus.CONTRACT_WAIT_FILL.getCode());
                pmDemands = pmDemandMapper.selectList(queryWrapper);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return AjaxResult.success(pmDemands.size());
    }

    /**
     * 查询需求单位待办事项(计划待提交)
     *
     * @return
     */
    @Override
    public List<PlanQuarterResponseVo> planWaitCommit() {
        return getListByProjectStatus(ProjectStatus.PLANWAITCOMMIT.getCode());
    }

    /**
     * 采购管理部门首页待办事项(计划待审核)
     *
     * @return
     */
    @Override
    public List<PlanQuarterResponseVo> planWaitExamine() {
        return getListByProjectStatus(ProjectStatus.PLANWAIEXAMINE.getCode());
    }

    /**
     * 需求单位待办事项
     */
    @Override
    public AjaxResult demandUnitWillManipulateTotal() {
        List<HomeToDoQueryResVo> homeRes = new ArrayList<>();
        //计划待提交，需求待提交，合同待填制，项目建设待完成
        List<PlanQuarterResponseVo> planRes = getListByProjectStatus(ProjectStatus.PLANWAITCOMMIT.getCode());
        List<PmDemandResVo> demandRes = demandService.demandWaitCommit();
        demandRes.addAll(demandService.contractWaitFilled());
        demandRes.addAll(demandService.projectWaitFinish());
        //转为首页table展示数据
        if (!ObjectUtils.isEmpty(planRes) && planRes.size() > 0) {
            for (PlanQuarterResponseVo planVo : planRes) {
                HomeToDoQueryResVo home = new HomeToDoQueryResVo();
                BeanUtils.copyProperties(planVo, home);
                homeRes.add(home);
            }
        }
        if (!ObjectUtils.isEmpty(demandRes) && demandRes.size() > 0) {
            for (PmDemandResVo demandResVo : demandRes) {
                HomeToDoQueryResVo home = new HomeToDoQueryResVo();
                BeanUtils.copyProperties(demandResVo, home);
                homeRes.add(home);
            }
        }

        return AjaxResult.success(homeRes);
    }

    @Override
    public AjaxResult purchasingManagementWillManipulateTotal() {
        List<HomeToDoQueryResVo> homeRes = new ArrayList<>();
        //计划待审核，需求待审核，任务待下达
        List<PlanQuarterResponseVo> planRes = getListByProjectStatus(ProjectStatus.PLANWAIEXAMINE.getCode());
        List<PmDemandResVo> demandRes = demandService.demandWaitExamine();
        demandRes.addAll(demandService.taskWaitRelease());
        //转为首页table展示数据
        if (!ObjectUtils.isEmpty(planRes) && planRes.size() > 0) {
            for (PlanQuarterResponseVo planVo : planRes) {
                HomeToDoQueryResVo home = new HomeToDoQueryResVo();
                BeanUtils.copyProperties(planVo, home);
                homeRes.add(home);
            }
        }
        if (!ObjectUtils.isEmpty(demandRes) && demandRes.size() > 0) {
            for (PmDemandResVo demandResVo : demandRes) {
                HomeToDoQueryResVo home = new HomeToDoQueryResVo();
                BeanUtils.copyProperties(demandResVo, home);
                homeRes.add(home);
            }
        }

        return AjaxResult.success(homeRes);
    }

    /**
     * 查询已被催告发函的计划
     *
     * @return
     */
    @Override
    public List<PlanQuarterResponseVo> willSendLetter() {
        LambdaQueryWrapper<PlanYears> yearsLp = new LambdaQueryWrapper<PlanYears>();
        yearsLp.eq(PlanYears::getDelFlay, DataIsDelete.DataNOTDelete);
        yearsLp.eq(PlanYears::getSendLetter, "1");
        List<PlanYears> planYears = planYearsMapper.selectList(yearsLp);

        LambdaQueryWrapper<PlanQuarter> quarterLp = new LambdaQueryWrapper<PlanQuarter>();
        quarterLp.eq(PlanQuarter::getDelFlay, DataIsDelete.DataNOTDelete);
        quarterLp.eq(PlanQuarter::getSendLetter, "1");
        List<PlanQuarter> planQuarters = planQuarterMapper.selectList(quarterLp);
        //融合一起
        List<PlanQuarterResponseVo> vos = planComplex(planYears, planQuarters);
        return vos;
    }

    public List<PlanQuarterResponseVo> getListByProjectStatus(String projectStatus) {
        LambdaQueryWrapper<PlanYears> yearsLp = new LambdaQueryWrapper<PlanYears>();
        yearsLp.eq(PlanYears::getDelFlay, DataIsDelete.DataNOTDelete);
        yearsLp.eq(PlanYears::getProjectStatus, projectStatus);
        List<PlanYears> planYears = planYearsMapper.selectList(yearsLp);

        LambdaQueryWrapper<PlanQuarter> quarterLp = new LambdaQueryWrapper<PlanQuarter>();
        quarterLp.eq(PlanQuarter::getDelFlay, DataIsDelete.DataNOTDelete);
        quarterLp.eq(PlanQuarter::getProjectStatus, projectStatus);
        List<PlanQuarter> planQuarters = planQuarterMapper.selectList(quarterLp);
        //融合一起
        List<PlanQuarterResponseVo> vos = planComplex(planYears, planQuarters);
        return vos;
    }

    /**
     * 年度计划与季度数据一起
     *
     * @return
     */
    public List<PlanQuarterResponseVo> planComplex(List<PlanYears> planYears, List<PlanQuarter> planQuarters) {
        List<PlanYearsResponseVo> yearsRes = changeTo(planYears);
        List<PlanQuarterResponseVo> quarterRes = quartzServiceImpl.changeTo(planQuarters);
        if (!ObjectUtils.isEmpty(yearsRes) && yearsRes.size() > 0) {
            for (PlanYearsResponseVo yearsRe : yearsRes) {
                PlanQuarterResponseVo responseVo = new PlanQuarterResponseVo();

                BeanUtils.copyProperties(yearsRe, responseVo);
                quarterRes.add(responseVo);
            }
        }
        return quarterRes;
    }

    //判断是否为超额计划
    public String isExcessOrNo(String projectType, BigDecimal evaluation) {
        BigDecimal threshold = new BigDecimal(0);
        //获取各个项目类型设定的概算金额阈值
        List<SysDictData> data = dictTypeService.selectDictDataByType("sys_over_limit_threshold");
        HashMap<String, String> thresholdMap = new LinkedHashMap<>();
        //各个类型的概算金额阈值
        for (SysDictData dictData : data) {
            //    类型----阈值
            thresholdMap.put(dictData.getDictLabel(), dictData.getDictValue());
        }
        //项目类型
        for (ProjectTypes value : ProjectTypes.values()) {
            if (projectType.equals(value.getCode())) {
                threshold = BigDecimal.valueOf(Long.parseLong(thresholdMap.get(value.getInfo())));
            }
            break;
        }
        if (evaluation.compareTo(threshold) == 1) {
            //是超额计划
            return ("1");
        } else {
            return ("0");
        }
    }

    //字段赋值对应的名称
    public List<PlanYearsResponseVo> changeTo(List<PlanYears> planYears) {
        //获取年度计划提报时间的阈值
        List<SysDictData> supTime = dictTypeService.selectDictDataByType("sys_annual_plan");
        HashMap<String, String> thresholdMap = new LinkedHashMap<>();
        //年度提报时间的阈值
        for (SysDictData dictData : supTime) {
            //    字段名称----阈值
            thresholdMap.put(dictData.getDictLabel(), dictData.getDictValue());
        }
        //提前或延后*天进行提醒
        List<SysDictData> alertTime = dictTypeService.selectDictDataByType("alert_time_setting");
        HashMap<String, String> alertTimeMap = new LinkedHashMap<>();
        //根据时间类别设定的提醒时间
        for (SysDictData dictData : alertTime) {
            //    字段名称----阈值
            alertTimeMap.put(dictData.getDictLabel(), dictData.getDictValue());
        }
        //获取字典数据
        HashMap<String, HashMap<String, String>> planEnums = dictTypeService.getAboutEnums();
        HashMap<String, String> projectTypesMap = planEnums.get("projectTypes");
        HashMap<String, String> planPurchaseModesMap = planEnums.get("planPurchaseModes");
        HashMap<String, String> projectAttributes = planEnums.get("projectAttributes");
        HashMap<String, String> purchaseServices = planEnums.get("purchaseServices");
        List<PlanYearsResponseVo> list = new ArrayList<PlanYearsResponseVo>();
        for (PlanYears planYear : planYears) {
            PlanYearsResponseVo responseVo = new PlanYearsResponseVo();
            SysDeptResponseVo sysDeptResponseVo = (SysDeptResponseVo) deptService.selectDeptById(Long.valueOf(planYear.getPurchaseDeptId())).get("sysDept");
            planYear.setPurchaseDeptName(sysDeptResponseVo.getDeptName());

            for (Map.Entry<String, String> entry : projectTypesMap.entrySet()) {
                if (planYear.getProjectType().equals(entry.getValue())) {
                    planYear.setProjectTypeStr(entry.getKey());
                    break;
                }
            }
            for (Map.Entry<String, String> entry : planPurchaseModesMap.entrySet()) {
                if (planYear.getPurchaseMode().equals(entry.getValue())) {
                    planYear.setPurchaseModeStr(entry.getKey());
                    break;
                }
            }
            //项目属性是拼接的
            if (!ObjectUtils.isEmpty(planYear.getProjectAttr())) {
                if (planYear.getProjectAttr().length() > 1) {
                    StringBuilder builder = new StringBuilder();
                    String[] split = planYear.getProjectAttr().split(",");
                    for (String s : split) {
                        for (Map.Entry<String, String> entry : projectAttributes.entrySet()) {
                            if (s.equals(entry.getValue())) {
                                if (!planYear.getProjectAttr().endsWith(s)) {
                                    builder.append(entry.getKey() + ",");
                                } else {
                                    builder.append(entry.getKey());
                                }
                                break;
                            }
                        }
                    }
                    planYear.setProjectAttrStr(builder.toString());
                } else {
                    for (Map.Entry<String, String> entry : projectAttributes.entrySet()) {
                        if (planYear.getProjectAttr().equals(entry.getValue())) {
                            planYear.setProjectAttrStr(entry.getKey());
                            break;
                        }
                    }
                }
            }
            for (Map.Entry<String, String> entry : purchaseServices.entrySet()) {
                if (planYear.getPurchaseServices().equals(entry.getValue())) {
                    planYear.setPurchaseServicesStr(entry.getKey());
                    break;
                }
            }
            BeanUtils.copyProperties(planYear, responseVo);
            try {
                responseVo.setTipsMessage(planYearsGetTips(responseVo, thresholdMap, alertTimeMap));
            } catch (ParseException e) {
                e.printStackTrace();
            }
            list.add(responseVo);
        }
        return list;
    }

    /**
     * 根据年度计划提报时间的阈值进行提示
     *
     * @return
     */
    public String planYearsGetTips(PlanYearsResponseVo responseVo, HashMap<String, String> thresholdMap, HashMap<String, String> alertTimeMap) throws ParseException {
        //提示信息：定义距离结束时间*天内提示
        int alertDay = Integer.parseInt(alertTimeMap.get("计划提报时间"));
        Integer remindTime = 1000 * 60 * 60 * 24 * alertDay;
        //提报时间的结束时间阈值
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date thresholdTime = dateFormat.parse(thresholdMap.get("起止时间结束"));
        Date date = new Date();
        thresholdTime.setYear(date.getYear());
        double surplus = thresholdTime.getTime() - date.getTime();
        //少于设定天数便提醒
        if (surplus < remindTime && surplus > 0) {
            //剩余天数(向上取整)
            int i = new Double(Math.ceil(surplus / 1000 / 60 / 60 / 24)).intValue();
            if (i > 0) {
                return "距离计划提报时间不足" + i + "天";
            }
        }
        //距离计划完成采购时间一周内提醒
        Integer alertTime = 1000 * 60 * 60 * 24 * 7;
        double surplusTwo = responseVo.getPlanPurchaseFinishTime().getTime() - System.currentTimeMillis();
        //少于设定阈值便提醒
        if (surplusTwo < alertTime && surplusTwo > 0) {
            //剩余天数(向上取整)
            int i = new Double(Math.ceil(surplusTwo / 1000 / 60 / 60 / 24)).intValue();
            if (i > 0) {
                return "距离计划完成采购时间不足" + i + "天";
            }
        }

        return "";
    }

}
